package ru.landgrafhomyak.serdha.api.v0.queries

import ru.landgrafhomyak.serdha.api.v0.misc.Column
import ru.landgrafhomyak.serdha.api.v0.misc.DatabaseType
import ru.landgrafhomyak.serdha.api.v0.misc.DatabaseTypesProvider
import ru.landgrafhomyak.serdha.api.v0.misc.QueryParam
import ru.landgrafhomyak.serdha.api.v0.runtime.InputRow
import ru.landgrafhomyak.serdha.api.v0.table.RowId
import ru.landgrafhomyak.serdha.api.v0.table.Table

public interface DeleteQuery<QueryUserExtension : Any> : _Query<QueryUserExtension> {
	public interface SingleRow<QueryUserExtension : Any> : DeleteQuery<QueryUserExtension>, _Query.Params2Table<QueryUserExtension> {
		public interface Creator<QueryUserExtension : Any, TargetTableUserExtension : Any> {
			public fun createDeleteSingleRowWithReturning(table: Table<TargetTableUserExtension, *>, creator: Scope<QueryUserExtension, TargetTableUserExtension>): QueryUserExtension

			public interface Scope<QueryUserExtension : Any, TargetTableUserExtension : Any> {
				public val types: DatabaseTypesProvider
				public fun <RT, DT : DatabaseType<RT>> param(name: String, type: DT): QueryParam<RT, DT, QueryUserExtension>

				public var rowToRemove: QueryParam<*, DatabaseType.ROW_ID<TargetTableUserExtension>, QueryUserExtension>

				public fun <RT, DT : DatabaseType<RT>> returnRemovedValue(name: String, column: Column<RT, DT, TargetTableUserExtension>): Column<RT, DT, QueryUserExtension>
				public fun <RT, DT : DatabaseType<RT>> returnParam(name: String, column: QueryParam<RT, DT, TargetTableUserExtension>): Column<RT, DT, QueryUserExtension>

				public fun ifNoRowSkip()
				public fun ifNoRowJustReturn()
				public fun ifNoRowThrow()
			}
		}
	}

	public interface FromSubquery<QueryUserExtension : Any> : DeleteQuery<QueryUserExtension>, _Query.Params2Table<QueryUserExtension> {
		public interface Creator<QueryUserExtension : Any, TargetTableUserExtension : Any, SelectorTableUserExtension : Any> {
			public fun createDeleteFromSubqueryWithReturning(table: Table<TargetTableUserExtension, *>, creator: Scope<QueryUserExtension, TargetTableUserExtension, SelectorTableUserExtension>): QueryUserExtension

			public interface Scope<QueryUserExtension : Any, TargetTableUserExtension : Any, SelectorTableUserExtension : Any> {
				public val types: DatabaseTypesProvider
				public fun <RT, DT : DatabaseType<RT>> param(name: String, type: DT): QueryParam<RT, DT, QueryUserExtension>

				public val selectorParams: InputRow.WithRedirect<SelectorTableUserExtension, QueryUserExtension>
				public var selectorColumn: Column<*, DatabaseType.ROW_ID<TargetTableUserExtension>, SelectorTableUserExtension>

				public fun <RT, DT : DatabaseType<RT>> returnFromSubquery(name: String, column: Column<RT, DT, SelectorTableUserExtension>): Column<RT, DT, QueryUserExtension>
				public fun <RT, DT : DatabaseType<RT>> returnRemovedValue(name: String, column: Column<RT, DT, TargetTableUserExtension>): Column<RT, DT, QueryUserExtension>
				public fun <RT, DT : DatabaseType<RT>> returnParam(name: String, column: QueryParam<RT, DT, TargetTableUserExtension>): Column<RT, DT, QueryUserExtension>

				public fun ifNoRowSkip()
				public fun ifNoRowJustReturn()
				public fun ifNoRowThrow()
			}
		}
	}

	public interface MultipleRows<QueryUserExtension : Any> : DeleteQuery<QueryUserExtension>, _Query.Table2Void<QueryUserExtension> {
		public interface Creator<QueryUserExtension : Any, TargetTableUserExtension : Any> {
			public fun createMultipleRows(table: Table<TargetTableUserExtension, *>, creator: Scope<QueryUserExtension, TargetTableUserExtension>): QueryUserExtension

			public interface Scope<QueryUserExtension : Any, TargetTableUserExtension : Any> {
				public val rowsToRemove: QueryParam<RowId<TargetTableUserExtension>, DatabaseType.ROW_ID<TargetTableUserExtension>, QueryUserExtension>

				public fun ifNoRowSkip()
				public fun ifNoRowThrow()
			}
		}
	}
}